@@ -18,14 +18,11 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-# BIP-0049 specifications:
-# https://github.com/bitcoin/bips/blob/master/bip-0049.mediawiki
 
 # Imports
-import binascii
-from .bip32             import Bip32
-from .bip44_base        import Bip44Base, Bip44Coins
-from .bip49_coin_helper import *
+from .bip32_utils import Bip32Utils
+from .bip44_base  import Bip44Base, Bip44Coins
+from .bip49_coins import *
 
 
 class Bip49Const:
@@ -34,7 +31,7 @@ class Bip49Const:
     # Specification name
     SPEC_NAME = "BIP-0049"
     # Purpose
-    PURPOSE   = Bip32.HardenIndex(49)
+    PURPOSE   = Bip32Utils.HardenIndex(49)
     # Allowed coins
     ALLOWED_COINS = \
         [
@@ -43,103 +40,125 @@ class Bip49Const:
             Bip44Coins.DOGECOIN, Bip44Coins.DOGECOIN_TESTNET,
             Bip44Coins.DASH    , Bip44Coins.DASH_TESTNET,
         ]
-    # Map from Bip44Coins to helper classes
-    COIN_TO_HELPER = \
+    # Map from Bip44Coins to coin classes
+    COIN_TO_CLASS = \
         {
-            Bip44Coins.BITCOIN  : BitcoinHelper,
-            Bip44Coins.LITECOIN : LitecoinHelper,
-            Bip44Coins.DOGECOIN : DogecoinHelper,
-            Bip44Coins.DASH     : DashHelper,
+            Bip44Coins.BITCOIN          : Bip49BitcoinMainNet,
+            Bip44Coins.BITCOIN_TESTNET  : Bip49BitcoinTestNet,
+            Bip44Coins.LITECOIN         : Bip49LitecoinMainNet,
+            Bip44Coins.LITECOIN_TESTNET : Bip49LitecoinTestNet,
+            Bip44Coins.DOGECOIN         : Bip49DogecoinMainNet,
+            Bip44Coins.DOGECOIN_TESTNET : Bip49DogecoinTestNet,
+            Bip44Coins.DASH             : Bip49DashMainNet,
+            Bip44Coins.DASH_TESTNET     : Bip49DashTestNet,
         }
 
 
 class Bip49(Bip44Base):
-    """ BIP49 class. """
+    """ BIP49 class. It allows master key generation and children keys derivation in according to BIP-0049.
+    BIP-0049 specifications: https://github.com/bitcoin/bips/blob/master/bip-0049.mediawiki
+    """
+
+    #
+    # Override methods
+    #
 
     def Purpose(self):
         """ Derive a child key from the purpose and return a new Bip object (e.g. BIP44, BIP49, BIP84).
         It calls the underlying _PurposeGeneric method with the current object as parameter.
-        Bip44DepthError is raised is chain depth is not suitable for deriving keys.
-        Bip32KeyError is raised (by Bip32) if the purpose results in an invalid key.
 
-        Returns (Bip object):
-            Bip object
+        Returns:
+            Bip49 object: Bip49 object
+
+        Raises:
+            Bip44DepthError: If current depth is not suitable for deriving keys
+            Bip32KeyError: If the derivation results in an invalid key
         """
         return self._PurposeGeneric(self)
 
     def Coin(self):
         """ Derive a child key from the coin type specified at construction and return a new Bip object (e.g. BIP44, BIP49, BIP84).
         It calls the underlying _CoinGeneric method with the current object as parameter.
-        Bip44DepthError is raised is chain depth is not suitable for deriving keys.
-        Bip32KeyError is raised (by Bip32) if the purpose results in an invalid key.
 
-        Returns (Bip object):
-            Bip object
+        Returns:
+            Bip49 object: Bip49 object
+
+        Raises:
+            Bip44DepthError: If current depth is not suitable for deriving keys
+            Bip32KeyError: If the derivation results in an invalid key
         """
         return self._CoinGeneric(self)
 
     def Account(self, acc_idx):
         """ Derive a child key from the specified account index and return a new Bip object (e.g. BIP44, BIP49, BIP84).
         It calls the underlying _AccountGeneric method with the current object as parameter.
-        Bip44DepthError is raised is chain depth is not suitable for deriving keys.
-        Bip32KeyError is raised (by Bip32) if the purpose results in an invalid key.
 
         Args:
-            acc_idx (int) : account index
+            acc_idx (int): Account index
 
-        Returns (Bip object):
-            Bip object
+        Returns:
+            Bip49 object: Bip49 object
+
+        Raises:
+            Bip44DepthError: If current depth is not suitable for deriving keys
+            Bip32KeyError: If the derivation results in an invalid key
         """
         return self._AccountGeneric(self, acc_idx)
 
     def Change(self, change_idx):
         """ Derive a child key from the specified account index and return a new Bip object (e.g. BIP44, BIP49, BIP84).
         It calls the underlying _ChangeGeneric method with the current object as parameter.
-        TypeError is raised if chain type is not a Bip44Changes enum.
-        Bip44DepthError is raised is chain depth is not suitable for deriving keys.
-        Bip32KeyError is raised (by Bip32) if the change results in an invalid key.
 
         Args:
-            change_idx (Bip44Changes) : change index, must a Bip44Changes enum
+            change_idx (Bip44Changes): Change index, must a Bip44Changes enum
 
-        Returns (Bip object):
-            Bip object
+        Returns:
+            Bip49 object: Bip49 object
+
+        Raises:
+            TypeError: If chain index is not a Bip44Changes enum
+            Bip44DepthError: If current depth is not suitable for deriving keys
+            Bip32KeyError: If the derivation results in an invalid key
         """
         return self._ChangeGeneric(self, change_idx)
 
     def AddressIndex(self, addr_idx):
         """ Derive a child key from the specified account index and return a new Bip object (e.g. BIP44, BIP49, BIP84).
         It calls the underlying _AddressIndexGeneric method with the current object as parameter.
-        Bip44DepthError is raised is chain depth is not suitable for deriving keys.
-        Bip32KeyError is raised (by Bip32) if the purpose results in an invalid key.
 
         Args:
-            addr_idx (int) : address index
+            addr_idx (int): Address index
 
-        Returns (Bip object):
-            Bip object
+        Returns:
+            Bip49 object: Bip49 object
+
+        Raises:
+            Bip44DepthError: If current depth is not suitable for deriving keys
+            Bip32KeyError: If the derivation results in an invalid key
         """
         return self._AddressIndexGeneric(self, addr_idx)
 
     @staticmethod
     def SpecName():
-        """ Get specification name
+        """ Get specification name.
 
-        Returns (str):
-            Specification name
+        Returns:
+            str: Specification name
         """
         return Bip49Const.SPEC_NAME
 
     @staticmethod
     def IsCoinAllowed(coin_idx):
         """ Get if the specified coin is allowed.
-        TypeError is raised if coin_idx is not of Bip44Coins enum.
 
         Args:
-            coin_idx (Bip44Coins) : coin index, must be a Bip44Coins enum
+            coin_idx (Bip44Coins): Coin index, must be a Bip44Coins enum
 
-        Returns (bool):
-            True if allowed, false otherwise
+        Returns :
+            bool: True if allowed, false otherwise
+
+        Raises:
+            TypeError: If coin_idx is not of Bip44Coins enum
         """
         if not isinstance(coin_idx, Bip44Coins):
             raise TypeError("Coin index is not an enumerative of Bip44Coins")
@@ -150,67 +169,19 @@ class Bip49(Bip44Base):
     def _GetPurpose():
         """ Get purpose.
 
-        Returns (int):
-            Purpose
+        Returns:
+            int: Purpose index
         """
         return Bip49Const.PURPOSE
 
     @staticmethod
-    def _GetMainNetVersions(coin_idx):
-        """ Get main net versions.
+    def _GetCoinClass(coin_idx):
+        """ Get coin class.
 
         Args:
-            coin_idx (Bip44Coins) : coin index, must be a Bip44Coins enum
+            coin_idx (Bip44Coins): Coin index, must be a Bip44Coins enum
 
-        Returns (dict):
-            Main net versions (main net at key "main", test net at key "test")
+        Returns:
+            BipCoinBase child object: BipCoinBase child object
         """
-        return Bip49Const.COIN_TO_HELPER[coin_idx].GetMainNetVersions()
-
-    @staticmethod
-    def _GetTestNetVersions(coin_idx):
-        """ Get test net versions.
-
-        Args:
-            coin_idx (Bip44Coins) : coin index, must be a Bip44Coins enum
-
-        Returns (dict):
-            Main net versions (main net at key "main", test net at key "test")
-        """
-        return Bip49Const.COIN_TO_HELPER[coin_idx].GetTestNetVersions()
-
-    @staticmethod
-    def _GetComputeAddressFct(coin_idx):
-        """ Compute compute address function.
-
-        Args:
-            coin_idx (Bip44Coins) : coin index, must be a Bip44Coins enum
-
-        Returns (function):
-            Compute address function
-        """
-        return Bip49Const.COIN_TO_HELPER[coin_idx].ComputeAddress
-
-    @staticmethod
-    def _GetWifNetVersions(coin_idx):
-        """ Get WIF net versions.
-
-        Args:
-            coin_idx (Bip44Coins) : coin index, must be a Bip44Coins enum
-
-        Returns (dict or None):
-            WIF net versions (main net at key "main", test net at key "test"), None if not supported
-        """
-        return Bip49Const.COIN_TO_HELPER[coin_idx].GetConfig().WIF_NET_VER
-
-    @staticmethod
-    def _GetCoinNames(coin_idx):
-        """ Get coin names.
-
-        Args:
-            coin_idx (Bip44Coins) : coin index, must be a Bip44Coins enum
-
-        Returns (dict):
-            Coin names (name at key "name", abbreviation at key "abbr")
-        """
-        return Bip49Const.COIN_TO_HELPER[coin_idx].GetConfig().NAMES
+        return Bip49Const.COIN_TO_CLASS[coin_idx]
